<?php
/**
 * Implements hook_media_browser_plugin_info().
 */
function qbank_dam_media_browser_plugin_info() {
  if (!(_check_config())) {
    return;
  }

  $plugins['qbank_file'] = array(
    'title' => t('QBank Library'),
    'class' => 'QBankMediaBrowser',
// Support for Media 1.x browser plugin API.
    '#title' => t('QBank Library'),
  );
  return $plugins;
}

/**
 * Provides a form for adding media items from QBank Library.
 *
 * @see qbank_file_add_form_submit()
 */
function qbank_file_add_form($form, &$form_state, array $options = array()) {
  $form['#attached']['js'][] = array(
    'type' => 'external',
    'data' => variable_get('qbank_api_protocol',
        'http') . '://' . variable_get('qbank_api_url') . '/connector/qbank-connector.min.js',
  );

  $modulePath = drupal_get_path('module', 'qbank_dam');
  $form['#attached']['js'][] = array(
    'type' => 'file',
    'data' =>  $modulePath . '/js/qbank3_mediabrowser.js',
  );

  $html_id = 'qbank-media-browser-wrapper' . rand();

  $form['#attached']['js'][] = array(
    'type' => 'setting',
    'data' => array(
      'qbank' => array(
        'token' => _get_token(),
        'url' => variable_get('qbank_api_url'),
        'html_id' => $html_id,
        'deployment_site' => variable_get('qbank_deployment_site', 0),
        'protocol' => variable_get('qbank_api_protocol', 'http'),
        'modulePath' => $modulePath,
      )
    ),
  );

  $form['url'] = array(
    '#type' => 'hidden',
    '#title' => 'URL',
    '#attributes' => array(
      'class' => array('media-add-from-qbank-url'),
    ),
    '#maxlength' => 255, // Maximum length of {file_managed}.uri
    '#element_validate' => array('qbank_validate_url'),
    '#required' => TRUE,
  );

  $form['filename'] = array(
    '#type' => 'hidden',
    '#title' => 'filename',
    '#attributes' => array('class' => array('media-add-from-qbank-filename')),
  );

  $form['extension'] = array(
    '#type' => 'hidden',
    '#title' => 'extension',
    '#attributes' => array('class' => array('media-add-from-qbank-extension')),
  );

  $form['title'] = array(
    '#type' => 'hidden',
    '#title' => 'title',
    '#attributes' => array('class' => array('media-add-from-qbank-title')),
  );

  $form['media_id'] = array(
    '#type' => 'hidden',
    '#title' => 'media_id',
    '#attributes' => array('class' => array('media-add-from-qbank-media_id')),
  );

  $form['qbank_wrapper'] = array(
    '#theme' => 'qbank_connector_wrapper',
    '#html_id' => $html_id,
  );

  $form['actions'] = array('#type' => 'actions');
  $form['actions']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Submit'),
    '#attributes' => array(
      'style' => 'display:none',
    ),
  );

  return $form;
}

/**
 * Save a file record based on a QBank Library.
 *
 * @see qbank_file_add_form()
 * @see _fetch_from_qbank()
 */
function qbank_file_add_form_submit($form, &$form_state) {
  $uri = $url = $form_state['values']['url'];

  try {
    $file = _fetch_from_qbank($uri, $form_state['values']['media_id']);
  } catch (Exception $e) {
    form_set_error('url', $e->getMessage());
    return;
  }

  if (empty($file->fid)) {
    form_set_error('url',
      t('Unable to add file from URL %file.', array('%file' => $url)));
    return;
  }
  else {
    $form_state['file'] = $file;
  }

  if (drupal_valid_path('file/' . $file->fid . '/edit')) {
    $destination = array('destination' => 'admin/content/file');
    if (isset($_GET['destination'])) {
      $destination = drupal_get_destination();
      unset($_GET['destination']);
    }
    $form_state['redirect'] = array(
      'file/' . $file->fid . '/edit',
      array('query' => $destination)
    );
  }
  else {
    $form_state['redirect'] = 'admin/content/file';
  }
}