function setWrapper(hook, html_id){
    jQuery('#' + hook).append('<div id="' + html_id + '"></div>');
}

CKEDITOR.plugins.add('QBank', {
    init: function (editor) {
        if (!Drupal.settings.qbank) {
            return;
        }

        // Add Button
        editor.ui.addButton('QBank', {
            label: Drupal.t('Insert image from QBank'),
            command: 'QBank',
            icon: this.path + '../img/icon-ckeditor.png'
        });

        // Add Command
        editor.addCommand('QBank', {
            exec: function (editor) {
                wrapper = 'qbank-ckeditor-dinamic-' + editor.name;
                jQuery('#qbank-image-loading').remove();

                if (!(jQuery('#' + wrapper).length)) {
                    setWrapper(editor.name, wrapper);
                }

                Drupal.qbank.ckeditorPlugin.setup(editor, wrapper);
            }
        });
    }
});

(function ($) {
    Drupal.qbank = Drupal.qbank || {};
    Drupal.qbank.ckeditorPlugin = Drupal.qbank.ckeditorPlugin || {};

    /**
     * File uploaded handler.
     */
    Drupal.qbank.ckeditorPlugin.onFileSuccess = function (response, editor) {
        response = $.parseJSON(response);

        if (response.status) {
            var editorInstance = CKEDITOR.instances[editor.name];
            var selection = editorInstance.getSelection();
            var selectedText = selection ? selection.getNative().toString() : '';
            var insertedHtml = selectedText ? response.image_tag : response.image_tag + ' ';

            editorInstance.insertHtml(insertedHtml);
        }
        else {
            alert(response.data);
        }
    };

    Drupal.qbank.ckeditorPlugin.setup = function (editor, wrapper) {
        var showDialog = function (selector) {
            dialog_height = jQuery(window).height() - 100;
            dialog_width = jQuery(window).width() - 100;

            jQuery('#' + wrapper).dialog({
                title: "QBank browser",
                modal: true,
                width: dialog_width,
                height: dialog_height,
                resizable: true,
            });

            jQuery(selector).css({'overflow' : 'hidden'});
        };

        var mediaSelected = function (media, image) {
            tosend = image[0];
            tosend.title = media.name;
            tosend.media_id = media.mediaId;

            jQuery('#qbank-ckeditor-dinamic-' + editor.name).empty();
            jQuery('#qbank-ckeditor-dinamic-' + editor.name).append('<img id="qbank-image-loading" style="position:absolute;top:0;bottom:0;margin:auto;left: 0;right: 0;" src=/' + Drupal.settings.qbank.modulePath + '/img/ajax-loader.gif>');

            jQuery.post("/qbank/upload", tosend)
                .done(function(data) {
                    // console.log(data);
                    Drupal.qbank.ckeditorPlugin.onFileSuccess(data, editor);
                    jQuery('#' + wrapper).dialog('close');
                })
                .error(function() {
                    alert('Error uploading file, check your log')
                    jQuery('#' + wrapper).dialog('close');
                });
        };

        var qbcConfig = {
            deploymentSite: Drupal.settings.qbank.deployment_site,
            api: {
                host: Drupal.settings.qbank.url,
                access_token: Drupal.settings.qbank.token,
                protocol: Drupal.settings.qbank.protocol
            },
            gui: {
                basehref: Drupal.settings.qbank.protocol + '://' + Drupal.settings.qbank.url + '/connector/'
            }
        };

        QBC = new QBankConnector(qbcConfig);
        mediaPicker = new QBC.mediaPicker({
            container: '#' + wrapper,
            onSelect: mediaSelected,
            onReady:  showDialog('#' + wrapper),
            modules: {
                folders: true,
                categories: true,
                moodboards: true,
                settings: true,
                content: {
                    header: false,
                    toolbar: true,
                    details: true
                },
                imageTool: {
                    crop: true
                },
                detail: {
                    showUseButton: true
                },
                searchResult: {
                    showUseButton: false
                }
            }
        });

        jQuery(window).resize(function() {
            jQuery('#' + wrapper).css({
                'height': (jQuery(window).height() - 100) + 'px',
                'width': (jQuery(window).width() - 100) + 'px'
            });
        });
    };

})(jQuery);